package com.zhibao.crm.utils;

import java.io.ByteArrayOutputStream;
import java.security.KeyFactory;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.interfaces.RSAPrivateKey;
import java.security.interfaces.RSAPublicKey;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.X509EncodedKeySpec;
import javax.crypto.Cipher;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.lang3.StringUtils;

public class RSAUtils {

   public static int MAX_ENCRYPT_BLOCK = 117;
   public static int MAX_DECRYPT_BLOCK = 128;


   public static String decrypt(String keyText, String text) throws Exception {
      return decrypt(keyText, text, "UTF-8");
   }

   public static String decrypt(String keyText, String text, String charset) throws Exception {
      PKCS8EncodedKeySpec pkcs8EncodedKeySpec5 = new PKCS8EncodedKeySpec(Base64.decodeBase64(keyText));
      KeyFactory keyFactory = KeyFactory.getInstance("RSA");
      PrivateKey privateKey = keyFactory.generatePrivate(pkcs8EncodedKeySpec5);
      Cipher cipher = Cipher.getInstance("RSA");
      cipher.init(2, privateKey);
      byte[] encryptedData = StringUtils.isEmpty(charset)?Base64.decodeBase64(text.getBytes()):Base64.decodeBase64(text.getBytes(charset));
      int inputLen = encryptedData.length;
      ByteArrayOutputStream out = new ByteArrayOutputStream();
      int offSet = 0;

      for(int i = 0; inputLen - offSet > 0; offSet = i * MAX_DECRYPT_BLOCK) {
         byte[] cache;
         if(inputLen - offSet > MAX_DECRYPT_BLOCK) {
            cache = cipher.doFinal(encryptedData, offSet, MAX_DECRYPT_BLOCK);
         } else {
            cache = cipher.doFinal(encryptedData, offSet, inputLen - offSet);
         }

         out.write(cache, 0, cache.length);
         ++i;
      }

      byte[] decryptedData = out.toByteArray();
      out.close();
      return StringUtils.isEmpty(charset)?new String(decryptedData):new String(decryptedData, charset);
   }

   public static String encrypt(String keyText, String text) throws Exception {
      return encrypt(keyText, text, "UTF-8");
   }

   public static String encrypt(String keyText, String text, String charset) throws Exception {
      X509EncodedKeySpec x509EncodedKeySpec2 = new X509EncodedKeySpec(Base64.decodeBase64(keyText));
      KeyFactory keyFactory = KeyFactory.getInstance("RSA");
      PublicKey publicKey = keyFactory.generatePublic(x509EncodedKeySpec2);
      Cipher cipher = Cipher.getInstance("RSA");
      cipher.init(1, publicKey);
      byte[] data = StringUtils.isEmpty(charset)?text.getBytes():text.getBytes(charset);
      int inputLen = data.length;
      int offSet = 0;
      int i = 0;

      ByteArrayOutputStream out;
      for(out = new ByteArrayOutputStream(); inputLen - offSet > 0; offSet = i * MAX_ENCRYPT_BLOCK) {
         byte[] cache;
         if(inputLen - offSet > MAX_ENCRYPT_BLOCK) {
            cache = cipher.doFinal(data, offSet, MAX_ENCRYPT_BLOCK);
         } else {
            cache = cipher.doFinal(data, offSet, inputLen - offSet);
         }

         out.write(cache, 0, cache.length);
         ++i;
      }

      byte[] outByte = out.toByteArray();
      byte[] encryptedData = Base64.encodeBase64(outByte);
      out.close();
      return StringUtils.isEmpty(charset)?new String(encryptedData):new String(encryptedData, charset);
   }

   public static RSAUtils.RSAKeyPair generateKeyPair() throws NoSuchAlgorithmException {
      KeyPairGenerator keyPairGenerator = KeyPairGenerator.getInstance("RSA");
      keyPairGenerator.initialize(1024);
      KeyPair keyPair = keyPairGenerator.generateKeyPair();
      RSAPublicKey rsaPublicKey = (RSAPublicKey)keyPair.getPublic();
      RSAPrivateKey rsaPrivateKey = (RSAPrivateKey)keyPair.getPrivate();
      String publicKeyString = Base64.encodeBase64String(rsaPublicKey.getEncoded());
      String privateKeyString = Base64.encodeBase64String(rsaPrivateKey.getEncoded());
      System.out.println("PublicKey:" + publicKeyString);
      System.out.println("PrivateKey:" + privateKeyString);
      RSAUtils.RSAKeyPair rsaKeyPair = new RSAUtils.RSAKeyPair(publicKeyString, privateKeyString);
      return rsaKeyPair;
   }

   public static void main(String[] args) {
      try {
         generateKeyPair();
      } catch (Exception var2) {
         System.out.println(var2.getMessage());
      }

   }


   public static class RSAKeyPair {

      private String publicKey;
      private String privateKey;


      public RSAKeyPair(String publicKey, String privateKey) {
         this.publicKey = publicKey;
         this.privateKey = privateKey;
      }

      public String getPublicKey() {
         return this.publicKey;
      }

      public String getPrivateKey() {
         return this.privateKey;
      }
   }
}
